@@ -103,6 +103,8 @@ static vps_config config[] = {
 /*	Devices	*/
 {"DEVICES",	NULL, PARAM_DEVICES},
 {"DEVNODES",	NULL, PARAM_DEVNODES},
+{"PCI",		NULL, PARAM_PCI_ADD},
+{"",		NULL, PARAM_PCI_DEL},
 /*	fs param */
 {"VE_ROOT",	NULL, PARAM_ROOT},
 {"VE_PRIVATE",	NULL, PARAM_PRIVATE},
@@ -137,107 +139,9 @@ static vps_config config[] = {
 {"IOPRIO",	NULL, PARAM_IOPRIO},
 {"BOOTORDER",	NULL, PARAM_BOOTORDER},
 
-{NULL		,NULL, -1}
+{NULL,		NULL, -1}
 };
 
-static struct option set_opt[] = {
-{"save",	no_argument, NULL, PARAM_SAVE},
-{"force",	no_argument, NULL, PARAM_FORCE},
-{"applyconfig",	required_argument, NULL, PARAM_APPLYCONFIG},
-{"applyconfig_map",	required_argument, NULL, PARAM_APPLYCONFIG_MAP},
-{"reset_ub",	no_argument, NULL, PARAM_RESET_UB},
-{"iptables",	required_argument, NULL, PARAM_IPTABLES},
-/*	UB	*/
-{"kmemsize",	required_argument, NULL, PARAM_KMEMSIZE},
-{"lockedpages",	required_argument, NULL, PARAM_LOCKEDPAGES},
-{"privvmpages",	required_argument, NULL, PARAM_PRIVVMPAGES},
-{"shmpages",	required_argument, NULL, PARAM_SHMPAGES},
-{"numproc",	required_argument, NULL, PARAM_NUMPROC},
-{"physpages",	required_argument, NULL, PARAM_PHYSPAGES},
-{"vmguarpages",	required_argument, NULL, PARAM_VMGUARPAGES},
-{"oomguarpages",required_argument, NULL, PARAM_OOMGUARPAGES},
-{"numtcpsock",	required_argument, NULL, PARAM_NUMTCPSOCK},
-{"numflock",	required_argument, NULL, PARAM_NUMFLOCK},
-{"numpty",	required_argument, NULL, PARAM_NUMPTY},
-{"numsiginfo",	required_argument, NULL, PARAM_NUMSIGINFO},
-{"tcpsndbuf",	required_argument, NULL, PARAM_TCPSNDBUF},
-{"tcprcvbuf",	required_argument, NULL, PARAM_TCPRCVBUF},
-{"othersockbuf",required_argument, NULL, PARAM_OTHERSOCKBUF},
-{"dgramrcvbuf",	required_argument, NULL, PARAM_DGRAMRCVBUF},
-{"numothersock",required_argument, NULL, PARAM_NUMOTHERSOCK},
-{"numfile",	required_argument, NULL, PARAM_NUMFILE},
-{"dcachesize",	required_argument, NULL, PARAM_DCACHESIZE},
-{"numiptent",	required_argument, NULL, PARAM_NUMIPTENT},
-{"avnumproc",	required_argument, NULL, PARAM_AVNUMPROC},
-{"swappages",	required_argument, NULL, PARAM_SWAPPAGES},
-/*	Capability */
-{"capability",	required_argument, NULL, PARAM_CAP},
-/*	Network	*/
-{"ipadd",	required_argument, NULL, PARAM_IP_ADD},
-{"ip",		required_argument, NULL, PARAM_IP_ADD},
-{"ipdel",	required_argument, NULL, PARAM_IP_DEL},
-{"skip_arpdetect",no_argument, NULL, PARAM_SKIPARPDETECT},
-{"netdev_add",	required_argument, NULL, PARAM_NETDEV_ADD},
-{"netdev_del",	required_argument, NULL, PARAM_NETDEV_DEL},
-{"hostname",	required_argument, NULL, PARAM_HOSTNAME},
-{"nameserver",	required_argument, NULL, PARAM_NAMESERVER},
-{"searchdomain",required_argument, NULL, PARAM_SEARCHDOMAIN},
-{"userpasswd",	required_argument, NULL, PARAM_USERPW},
-/*	Devices	*/
-{"devices",	required_argument, NULL, PARAM_DEVICES},
-{"devnodes",	required_argument, NULL, PARAM_DEVNODES},
-/*	fs param */
-{"root",	required_argument, NULL, PARAM_ROOT},
-{"private",	required_argument, NULL, PARAM_PRIVATE},
-{"noatime",	required_argument, NULL, PARAM_NOATIME},
-/*	template	*/
-{"ostemplate",	required_argument, NULL, PARAM_OSTEMPLATE},
-{"pkgset",	required_argument, NULL, PARAM_PKGSET},
-{"pkgver",	required_argument, NULL, PARAM_PKGVER},
-/*	Cpu	*/
-{"cpuunits",	required_argument, NULL, PARAM_CPUUNITS},
-{"cpuweight",	required_argument, NULL, PARAM_CPUWEIGHT},
-{"cpulimit",	required_argument, NULL, PARAM_CPULIMIT},
-{"cpus",	required_argument, NULL, PARAM_VCPUS},
-/*	create param	*/
-{"onboot",	required_argument, NULL, PARAM_ONBOOT},
-{"setmode",	required_argument, NULL, PARAM_SETMODE},
-{"disabled",	required_argument, NULL, PARAM_DISABLED},
-/*	quota */
-{"diskspace",	required_argument, NULL, PARAM_DISKSPACE},
-{"diskinodes",	required_argument, NULL, PARAM_DISKINODES},
-{"quotatime",	required_argument, NULL, PARAM_QUOTATIME},
-{"quotaugidlimit", required_argument, NULL, PARAM_QUOTAUGIDLIMIT},
-{"meminfo",	required_argument, NULL, PARAM_MEMINFO},
-/*	veth	*/
-{"veth_add",	required_argument, NULL, PARAM_VETH_ADD_CMD},
-{"veth_del",	required_argument, NULL, PARAM_VETH_DEL_CMD},
-/*	netif	*/
-{"netif_add",	required_argument, NULL, PARAM_NETIF_ADD_CMD},
-{"netif_del",	required_argument, NULL, PARAM_NETIF_DEL_CMD},
-{"mac_filter",	required_argument, NULL, PARAM_NETIF_MAC_FILTER},
-
-{"mac",		required_argument, NULL, PARAM_NETIF_MAC},
-{"ifname",	required_argument, NULL, PARAM_NETIF_IFNAME},
-{"host_mac",	required_argument, NULL, PARAM_NETIF_HOST_MAC},
-{"host_ifname",	required_argument, NULL, PARAM_NETIF_HOST_IFNAME},
-{"bridge",	required_argument, NULL, PARAM_NETIF_BRIDGE},
-
-/*	name	*/
-{"name",	required_argument, NULL, PARAM_NAME},
-{"features",	required_argument, NULL, PARAM_FEATURES},
-{"ioprio",	required_argument, NULL, PARAM_IOPRIO},
-{"description",	required_argument, NULL, PARAM_DESCRIPTION},
-{"bootorder",	required_argument, NULL, PARAM_BOOTORDER},
-
-{NULL, 0, NULL, 0}
-};
-
-struct option *get_set_opt(void)
-{
-	return set_opt;
-}
-
 const vps_config *conf_get_by_name(const vps_config *conf, const char *name)
 {
 	const vps_config *p;
@@ -400,7 +304,7 @@ static int conf_store_ulong(list_head_t *conf, char *name, unsigned long *val)
 }
 
 /******************** Features *************************/
-static int parse_features(env_param *env, char *val)
+static int parse_features(env_param_t *env, char *val)
 {
 	int ret = 0;
 	char *token;
@@ -462,7 +366,7 @@ static int store_ioprio(vps_param *old_p, vps_param *vps_p,
 }
 
 /******************** Iptables *************************/
-static int parse_iptables(env_param *env, char *val)
+static int parse_iptables(env_param_t *env, char *val)
 {
 	char *token;
 	struct iptables_s *ipt;
@@ -476,13 +380,13 @@ static int parse_iptables(env_param *env, char *val)
 			ret = ERR_INVAL_SKIP;
 			continue;
 		}
-		env->ipt_mask |= (unsigned long)ipt->mask;
+		env->ipt_mask |= ipt->mask;
 	}
 
 	return ret;
 }
 
-static void store_iptables(unsigned long ipt_mask, vps_config *conf,
+static void store_iptables(unsigned long long ipt_mask, vps_config *conf,
 	list_head_t *conf_h)
 {
 	char buf[STR_SIZE];
@@ -497,7 +401,7 @@ static void store_iptables(unsigned long ipt_mask, vps_config *conf,
 static int store_env(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	list_head_t *conf_h)
 {
-	env_param *env = &vps_p->res.env;
+	env_param_t *env = &vps_p->res.env;
 
 	switch (conf->id) {
 	case PARAM_IPTABLES:
@@ -787,8 +691,8 @@ int check_ip_dot(char *ip)
 static int parse_ip(vps_param *vps_p, char *val, int id)
 {
 	char *token;
-	char dst[64];
-	unsigned int ip[16];
+	char dst[INET6_ADDRSTRLEN];
+	unsigned char ip[sizeof(struct in6_addr)];
 	int family;
 	net_param *net;
 
@@ -962,6 +866,9 @@ static int store_dq(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	dq_param *param = &vps_p->res.dq;
 
 	switch (conf->id) {
+	case PARAM_DISK_QUOTA:
+		conf_store_yesno(conf_h, conf->name, param->enable);
+		break;
 	case PARAM_DISKSPACE:
 		if (param->diskspace == NULL)
 			break;
@@ -1050,8 +957,11 @@ static int parse_dev(vps_param *vps_p, char *val)
 	for_each_strtok(token, val, " ") {
 		if (parse_devices_str(token, &dev))
 			return ERR_INVAL;
-		if (add_dev_param(&vps_p->res.dev, &dev))
+		if (add_dev_param(&vps_p->res.dev, &dev)) {
+			if (dev.name)
+				free(dev.name);
 			return ERR_NOMEM;
+		}
 	}
 
 	return 0;
@@ -1108,36 +1018,51 @@ static int store_dev(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 static int parse_devnodes_str(const char *str, dev_res *dev)
 {
 	char *ch;
-	unsigned int len;
-	char buf[64];
+	unsigned int len, buf_len;
+	char *buf;
 	struct stat st;
+	int ret = ERR_INVAL;
 
-	if ((ch = strchr(str, ':')) == NULL)
+	if ((ch = strrchr(str, ':')) == NULL)
 		return ERR_INVAL;
 	ch++;
 	len = ch - str;
-	if (len > sizeof(dev->name))
-		return ERR_INVAL;
 	memset(dev, 0, sizeof(*dev));
+
+	dev->name = malloc(len);
+	if (dev->name == NULL)
+		return ERR_NOMEM;
 	snprintf(dev->name, len, "%s", str);
-	snprintf(buf, sizeof(buf), "/dev/%s", dev->name);
+
+	buf_len = 5 + len;
+	buf = alloca(buf_len);
+	if (buf == NULL) {
+		ret = ERR_NOMEM;
+		goto err;
+	}
+	snprintf(buf, buf_len, "/dev/%s", dev->name);
 	if (stat(buf, &st)) {
 		logger(-1, errno, "Incorrect device name %s", buf);
-		return ERR_INVAL;
+		goto err;
 	}
+
 	if (S_ISCHR(st.st_mode))
 		dev->type = S_IFCHR;
 	else if (S_ISBLK(st.st_mode))
 		dev->type = S_IFBLK;
 	else {
 		logger(-1, 0, "The %s is not block or character device", buf);
-		return ERR_INVAL;
+		goto err;
 	}
 	dev->dev = st.st_rdev;
 	dev->type |= VE_USE_MINOR;
 	if (parse_dev_perm(ch, &dev->mask))
-		return ERR_INVAL;
+		goto err;
 	return 0;
+err:
+	free(dev->name);
+	dev->name = NULL;
+	return ret;
 }
 
 static int parse_devnodes(vps_param *vps_p, char *val)
@@ -1148,8 +1073,11 @@ static int parse_devnodes(vps_param *vps_p, char *val)
 	for_each_strtok(token, val, " ") {
 		if (parse_devnodes_str(token, &dev))
 			return ERR_INVAL;
-		if (add_dev_param(&vps_p->res.dev, &dev))
+		if (add_dev_param(&vps_p->res.dev, &dev)) {
+			if (dev.name)
+				free(dev.name);
 			return ERR_NOMEM;
+		}
 	}
 
 	return 0;
@@ -1194,6 +1122,65 @@ static int store_devnodes(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	return 0;
 }
 
+static int parse_pci(vps_param *vps_p, char *val, int id)
+{
+	int domain;
+	unsigned int bus;
+	unsigned int slot;
+	unsigned int func;
+	char *token;
+
+	int ret;
+	pci_param *pci;
+	char buf[64];
+
+	if (id == PARAM_PCI_ADD)
+		pci = &vps_p->res.pci;
+	else if (id == PARAM_PCI_DEL)
+		pci = &vps_p->del_res.pci;
+	else
+		return 0;
+
+	for_each_strtok(token, val, " ") {
+		ret = sscanf(token, "%x:%x:%x.%d", &domain, &bus, &slot, &func);
+		if (ret != 4) {
+			domain = 0;
+			ret = sscanf(token, "%x:%x.%d", &bus, &slot, &func);
+			if (ret != 3)
+				return ERR_INVAL;
+		}
+		snprintf(buf, sizeof(buf), "%04x:%02x:%02x.%d",
+						domain, bus, slot, func);
+		if (!find_str(&pci->list, buf))
+			add_str_param(&pci->list, buf);
+	}
+
+	return 0;
+}
+
+static int store_pci(vps_param *old_p, vps_param *vps_p, vps_config *conf,
+	list_head_t *conf_h)
+{
+	list_head_t pci;
+	int ret;
+
+	if (conf->id != PARAM_PCI_ADD)
+		return 0;
+
+	if (list_empty(&vps_p->res.pci.list) &&
+			list_empty(&vps_p->del_res.pci.list))
+		return 0;
+
+	list_head_init(&pci);
+	merge_str_list(0, &old_p->res.pci.list, &vps_p->res.pci.list,
+					&vps_p->del_res.pci.list, &pci);
+
+	ret = conf_store_strlist(conf_h, conf->name, &pci, 1);
+	free_str_param(&pci);
+
+	return ret;
+}
+
 static int store_misc(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	list_head_t *conf_h)
 {
@@ -1282,87 +1269,6 @@ static int store_cpu(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	return 0;
 }
 
-/******************** VETH *********************************/
-static int parse_veth_str(const char *str, veth_dev *dev, int operation)
-{
-	char *ch, *tmp;
-	int len, err;
-
-	memset(dev, 0, sizeof(*dev));
-	if (!operation) {
-		/* Removing veth device */
-		/* Parsing veth device name in CT0 */
-		if ((ch = strchr(str, ',')) != NULL)
-			return ERR_INVAL;
-		len = strlen(str) + 1;
-		if (len > IFNAMSIZE)
-			return ERR_INVAL;
-		snprintf(dev->dev_name, len, "%s", str);
-		return 0;
-	}
-	/* Creating veth device */
-	/* Parsing veth device name in CT0 */
-	if ((ch = strchr(str, ',')) == NULL)
-		return ERR_INVAL;
-	ch++;
-	len = ch - str;
-	if (len > IFNAMSIZE)
-		return ERR_INVAL;
-	snprintf(dev->dev_name, len, "%s", str);
-	tmp = ch;
-
-	/* Parsing veth MAC address in CT0 */
-	if ((ch = strchr(tmp, ',')) == NULL)
-		return ERR_INVAL;
-	len = ch - tmp;
-	if (len != 3*ETH_ALEN -1)
-		return ERR_INVAL;
-	dev->addrlen = ETH_ALEN;
-	err = parse_hwaddr(tmp, dev->dev_addr);
-	if (err)
-		return ERR_INVAL;
-	ch++;
-	tmp = ch;
-
-	/* Parsing veth name in CT */
-	if ((ch = strchr(tmp, ',')) == NULL)
-		return ERR_INVAL;
-	ch++;
-	len = ch - tmp;
-	if (len > IFNAMSIZE)
-		return ERR_INVAL;
-	snprintf(dev->dev_name_ve, len, "%s", tmp);
-
-	/* Parsing veth MAC address in CT */
-	len = strlen(ch);
-	if (len != 3*ETH_ALEN -1)
-		return ERR_INVAL;
-	dev->addrlen_ve = ETH_ALEN;
-	err = parse_hwaddr(ch, dev->dev_addr_ve);
-	if (err)
-		return ERR_INVAL;
-
-	return 0;
-}
-
-static int parse_veth(vps_param *vps_p, char *val, int operation)
-{
-	int ret;
-	char *token;
-	veth_dev dev;
-
-	for_each_strtok(token, val, " ") {
-		if (parse_veth_str(token, &dev, operation))
-			return ERR_INVAL;
-		if (operation)
-			ret = add_veth_param(&vps_p->res.veth, &dev);
-		else
-			ret = add_veth_param(&vps_p->del_res.veth, &dev);
-	}
-
-	return 0;
-}
-
 /********************* NETIF **********************************/
 
 static int parse_mac_filter_cmd (veth_dev *dev, char *str)
@@ -1966,12 +1872,6 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_DEF_OSTEMPLATE:
 		ret = conf_parse_str(&vps_p->res.tmpl.def_ostmpl, val);
 		break;
-	case PARAM_PKGSET:
-		ret = conf_parse_str(&vps_p->res.tmpl.pkgset, val);
-		break;
-	case PARAM_PKGVER:
-		ret = conf_parse_str(&vps_p->res.tmpl.pkgver, val);
-		break;
 	case PARAM_OSTEMPLATE:
 		ret = conf_parse_str(&vps_p->res.tmpl.ostmpl, val);
 		break;
@@ -1999,6 +1899,10 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_DEVICES:
 		ret = parse_dev(vps_p, val);
 		break;
+	case PARAM_PCI_ADD:
+	case PARAM_PCI_DEL:
+		ret = parse_pci(vps_p, val, id);
+		break;
 	case PARAM_DEVNODES:
 		ret = parse_devnodes(vps_p, val);
 		break;
@@ -2021,8 +1925,7 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_CPULIMIT:
 		if (vps_p->res.cpu.limit != NULL)
 			break;
-		if (parse_cpulimit(&vps_p->res.cpu.limit, val))
-			return ERR_INVAL;
+		ret = parse_cpulimit(&vps_p->res.cpu.limit, val);
 		break;
 	case PARAM_VCPUS:
 		ret = conf_parse_ulong(&vps_p->res.cpu.vcpus, val);
@@ -2031,29 +1934,13 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 		ret = parse_meminfo(&vps_p->res.meminfo, val);
 		break;
 	case PARAM_VETH_ADD:
-		if (!list_empty(&vps_p->res.veth.dev))
-			break;
-		ret = parse_veth(vps_p, val, 1);
-		vps_p->res.veth.version = 1;
-		break;
-	case PARAM_VETH_ADD_CMD:
-		logger(0, 0, "Warning: --veth_add option is deprecated, use"
-			" --netif_add instead");
-		ret = parse_veth(vps_p, val, 1);
-		break;
-	case PARAM_VETH_DEL_CMD:
-		logger(0, 0, "Warning: --veth_del option is deprecated, use"
-			" --netif_del instead");
-		ret = parse_veth(vps_p, val, 0);
+		logger(0, 0, "Warning: VETH parameter is deprecated, remove "
+				"it and use --netif_add instead");
 		break;
 	case PARAM_NETIF_ADD:
-		/* Skip VETH parameter if NETIF exists */
-		if (vps_p->res.veth.version)
-			free_veth_param(&vps_p->res.veth);
 		if (!list_empty(&vps_p->res.veth.dev))
 			break;
 		ret = parse_netif(veid, &vps_p->res.veth, val);
-		vps_p->res.veth.version = 0;
 		break;
 	case PARAM_NETIF_ADD_CMD:
 		ret = parse_netif_cmd(veid, &vps_p->res.veth, val);
@@ -2129,6 +2016,7 @@ static int store(vps_param *old_p, vps_param *vps_p, list_head_t *conf_h)
 		store_dq(old_p, vps_p, conf, conf_h);
 		store_dev(old_p, vps_p, conf, conf_h);
 		store_devnodes(old_p, vps_p, conf, conf_h);
+		store_pci(old_p, vps_p, conf, conf_h);
 		store_misc(old_p, vps_p, conf, conf_h);
 		store_cpu(old_p, vps_p, conf, conf_h);
 		store_meminfo(old_p, vps_p, conf, conf_h);
@@ -2242,17 +2130,17 @@ static int write_conf(char *fname, list_head_t *head)
 
 	file = canonicalize_file_name(fname);
 	if (file == NULL) {
-		if (errno != ENOENT)
-		{
+		if (errno != ENOENT) {
 			logger(-1, errno, "Unable to resolve path %s", fname);
 			return 1;
 		}
-		file = strdup(fname);
+		file = vz_strdup(fname);
+		if (!file)
+			return ret;
 	}
 	tmpfile = vz_malloc(strlen(file) + strlen(suffix) + 1);
-	if (!tmpfile) {
+	if (!tmpfile)
 		goto out_file;
-	}
 	sprintf(tmpfile, "%s%s", file, suffix);
 	if ((fp = fopen(tmpfile, "w")) == NULL) {
 		logger(-1, errno, "Unable to create configuration"
@@ -2402,6 +2290,7 @@ vps_param *init_vps_param()
 	list_head_init(&param->res.misc.nameserver);
 	list_head_init(&param->res.misc.searchdomain);
 	list_head_init(&param->res.dev.dev);
+	list_head_init(&param->res.pci.list);
 	list_head_init(&param->res.veth.dev);
 
 	list_head_init(&param->del_res.net.ip);
@@ -2411,6 +2300,7 @@ vps_param *init_vps_param()
 	list_head_init(&param->del_res.misc.nameserver);
 	list_head_init(&param->del_res.misc.searchdomain);
 	list_head_init(&param->del_res.dev.dev);
+	list_head_init(&param->del_res.pci.list);
 	list_head_init(&param->del_res.veth.dev);
 	param->res.meminfo.mode = -1;
 	param->res.io.ioprio = -1;
@@ -2529,8 +2419,6 @@ static void free_tmpl(tmpl_param *tmpl)
 {
 	FREE_P(tmpl->def_ostmpl)
 	FREE_P(tmpl->ostmpl)
-	FREE_P(tmpl->pkgset)
-	FREE_P(tmpl->pkgver)
 	FREE_P(tmpl->dist)
 }
 
@@ -2560,6 +2448,11 @@ static void free_dev(dev_param *dev)
 	free_dev_param(dev);
 }
 
+static void free_pci(pci_param *pci)
+{
+	free_str_param(&pci->list);
+}
+
 static void free_cpu(cpu_param *cpu)
 {
 	FREE_P(cpu->units)
@@ -2596,6 +2489,7 @@ static void free_vps_res(vps_res *res)
 	free_net(&res->net);
 	free_misc(&res->misc);
 	free_dev(&res->dev);
+	free_pci(&res->pci);
 	free_cpu(&res->cpu);
 	free_dq(&res->dq);
 	free_cpt(&res->cpt);
@@ -2671,8 +2565,6 @@ static void merge_tmpl(tmpl_param *dst, tmpl_param *src)
 {
 	MERGE_STR(def_ostmpl)
 	MERGE_STR(ostmpl)
-	MERGE_STR(pkgset)
-	MERGE_STR(pkgver)
 	MERGE_STR(dist)
 }
 
@@ -2732,13 +2624,18 @@ static void merge_dev(dev_param *dst, dev_param *src)
 	}
 }
 
+static void merge_pci(pci_param *dst, pci_param *src)
+{
+	MERGE_LIST(list)
+}
+
 static void merge_cap(cap_param *dst, cap_param *src)
 {
 	MERGE_INT(on)
 	MERGE_INT(off)
 }
 
-static void merge_env(env_param *dst, env_param *src)
+static void merge_env(env_param_t *dst, env_param_t *src)
 {
 	MERGE_INT(veid)
 	MERGE_INT(ipt_mask)
@@ -2790,6 +2687,7 @@ static int merge_res(vps_res *dst, vps_res *src)
 	merge_cap(&dst->cap, &src->cap);
 	merge_dq(&dst->dq, &src->dq);
 	merge_dev(&dst->dev, &src->dev);
+	merge_pci(&dst->pci, &src->pci);
 	merge_env(&dst->env, &src->env);
 	merge_cpt(&dst->cpt, &src->cpt);
 	merge_meminfo(&dst->meminfo, &src->meminfo);
@@ -2809,7 +2707,6 @@ int merge_vps_param(vps_param *dst, vps_param *src)
 
 int merge_global_param(vps_param *dst, vps_param *src)
 {
-	MERGE_INT(res.dq.enable)
 	MERGE_INT(res.net.ipv6_net)
 	MERGE_INT(del_res.net.ipv6_net)
 	return 0;
