@@ -272,6 +272,31 @@ int vz_setluid(envid_t veid)
 	return 0;
 }
 
+static int vz_env_configure(int fd, envid_t veid, const char *osrelease)
+{
+	int ret = 0;
+	struct vzctl_ve_configure *cparam;
+	int len;
+
+	len = strlen(osrelease) + 1;
+	cparam = calloc(1, sizeof(struct vzctl_ve_configure) + len);
+	if (cparam == NULL)
+		return VZ_RESOURCE_ERROR;
+
+	cparam->veid = veid;
+	cparam->key = VE_CONFIGURE_OS_RELEASE;
+	cparam->size = len;
+	strcpy(cparam->data, osrelease);
+
+	if (ioctl(fd, VZCTL_VE_CONFIGURE, cparam) != 0)
+		if (errno != ENOTTY)
+			ret = VZ_SET_OSRELEASE;
+
+	free(cparam);
+
+	return ret;
+}
+
 static int _env_create(vps_handler *h, envid_t veid, int wait_p, int err_p,
 	void *data)
 {
@@ -299,7 +324,7 @@ static int _env_create(vps_handler *h, envid_t veid, int wait_p, int err_p,
 	create_param.feature_mask = res->env.features_mask;
 	create_param.known_features = res->env.features_known;
 	/* sysfs enabled by default, unless explicitly disabled */
-	if (! res->env.features_known & VE_FEATURE_SYSFS) {
+	if (! (res->env.features_known & VE_FEATURE_SYSFS)) {
 		create_param.feature_mask |= VE_FEATURE_SYSFS;
 		create_param.known_features |= VE_FEATURE_SYSFS;
 	}
@@ -346,13 +371,18 @@ try:
 		}
 		goto env_err;
 	}
+	if (res->env.osrelease != NULL) {
+		ret = vz_env_configure(h->vzfd, veid,
+				res->env.osrelease);
+		if (ret != 0)
+			goto env_err;
+	}
+
 	close(h->vzfd);
 	/* Create /fastboot to skip run fsck */
 	fd = open("/fastboot", O_CREAT | O_RDONLY, 0644);
 	close(fd);
 
-	mk_reboot_script();
-
 	if (res->misc.wait == YES) {
 		if (add_reach_runlevel_mark()) {
 			ret = VZ_WAIT_FAILED;
@@ -414,7 +444,7 @@ static int vz_real_env_create(vps_handler *h, envid_t veid, vps_res *res,
 		logger(-1, errno, "Unable to fork");
 		return VZ_RESOURCE_ERROR;
 	} else if (pid == 0) {
-		if ((ret = vps_set_cap(veid, &res->cap)))
+		if ((ret = vps_set_cap(veid, &res->env, &res->cap)))
 			goto env_err;
 		if (fn == NULL) {
 			ret = _env_create(h, veid, wait_p, err_p, (void *)res);
@@ -448,6 +478,8 @@ int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
 	act.sa_flags = SA_NOCLDSTOP;
 	sigaction(SIGCHLD, &act, NULL);
 
+	get_osrelease(res);
+
 	if ((pid = fork()) < 0) {
 		logger(-1, errno, "Can not fork");
 		ret =  VZ_RESOURCE_ERROR;
@@ -473,7 +505,7 @@ int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
 	close(wait_p[0]);
 	close(err_p[1]);
 	ret = read(status_p[0], &errcode, sizeof(errcode));
-	if (ret) {
+	if (ret > 0) {
 		ret = errcode;
 		switch(ret) {
 		case VZ_NO_ACCES:
@@ -494,6 +526,10 @@ int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
 			logger(0, 0, "Unable to set"
 				" wait functionality");
 			break;
+		case VZ_SET_OSRELEASE:
+			logger(-1, 0, "Unable to set osrelease to %s",
+					res->env.osrelease);
+			break;
 		}
 	}
 err:
@@ -610,7 +646,8 @@ err:
 	free_dist_actions(&actions);
 	if (ret) {
 		/* Kill environment */
-		logger(-1, 0, "Container start failed");
+		logger(-1, 0, "Container start failed (try to check kernel "
+				"messages, e.g. \"dmesg | tail\")");
 		write(wait_p[1], &err, sizeof(err));
 	} else {
 		if (!read(err_p[0], &ret, sizeof(ret))) {
@@ -808,10 +845,10 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
 	if ((ret = env_stop(h, veid, res->fs.root, stop_mode)))
 		goto end;
 	mod_cleanup(h, veid, action, param);
-	vps_cleanup_res(h, veid, param, STATE_STOPPING);
+	/* Cleanup CT IPs */
+	run_net_script(veid, DEL, &param->del_res.net.ip,
+			STATE_STOPPING, param->res.net.skip_arpdetect);
 	ret = vps_umount(h, veid, res->fs.root, skip);
-	/* Clear CT network configuration */
-	ret = run_pre_script(veid, VPS_STOP);
 
 end:
 	free_str_param(&param->del_res.net.ip);
