@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -37,49 +37,46 @@
 #include "script.h"
 
 static volatile sig_atomic_t alarm_flag, child_exited;
-static char *argv_bash[] = {"bash", NULL};
-static char *envp_bash[] = {"HOME=/", "TERM=linux",
-	"PATH=/bin:/sbin:/usr/bin:/usr/sbin:.", NULL};
+static char *envp_bash[] = {"HOME=/", "TERM=linux", ENV_PATH, NULL};
 
 int vz_env_create_ioctl(vps_handler *h, envid_t veid, int flags);
 
 int execvep(const char *path, char *const argv[], char *const envp[])
 {
-	if (!strchr(path, '/')) {
-		char *p = getenv("PATH");
+	const char *p, *p2;
 
-		if (!p)
-			p = "/bin:/usr/bin:/sbin";
-		for (; p && *p;) {
-			char partial[FILENAME_MAX];
-			char *p2;
-
-			p2 = strchr(p, ':');
-			if (p2) {
-				size_t len = p2 - p;
-
-				strncpy(partial, p, len);
-				partial[len] = 0;
-			} else {
-				strcpy(partial, p);
-			}
-			if (strlen(partial))
-				strcat(partial, "/");
-			strcat(partial, path);
-
-			execve(partial, argv, envp != NULL ? envp : envp_bash);
-
-			if (errno != ENOENT)
-				return -1;
-			if (p2) {
-				p = p2 + 1;
-			} else {
-				p = 0;
-			}
-		}
-		return -1;
-	} else
+	if (strchr(path, '/'))
 		return execve(path, argv, envp);
+
+	for (p = DEF_PATH; p && *p; p = p2) {
+		char partial[FILENAME_MAX];
+		size_t res;
+
+		p2 = strchr(p, ':');
+		if (p2) {
+			size_t len = p2 - p;
+
+			strncpy(partial, p, len);
+			partial[len] = 0;
+			p2++;
+		} else {
+			strcpy(partial, p);
+		}
+		if (strlen(partial))
+			vz_strlcat(partial, "/", sizeof(partial));
+
+		res = vz_strlcat(partial, path, sizeof(partial));
+		if (res >= sizeof(partial)) {
+			errno = ENAMETOOLONG;
+			break;
+		}
+
+		execve(partial, argv, envp ? envp : envp_bash);
+
+		if (errno != ENOENT)
+			break;
+	}
+	return -1;
 }
 
 static int stdredir(int rdfd, int wrfd)
@@ -146,14 +143,15 @@ int env_wait(int pid)
 	return ret;
 }
 
-static int vps_real_exec(vps_handler *h, envid_t veid, char *root,
-	int exec_mode, char *const argv[], char *const envp[], char *std_in,
+static int vps_real_exec(vps_handler *h, envid_t veid, const char *root,
+	int exec_mode, char *argv[], char *const envp[], char *std_in,
 	int timeout)
 {
 	int ret, pid;
 	int in[2], out[2], err[2], st[2];
 	int fl = 0;
 	struct sigaction act;
+	char *def_argv[] = { NULL, NULL };
 
 	if (pipe(in) < 0 ||
 		pipe(out) < 0 ||
@@ -219,10 +217,12 @@ static int vps_real_exec(vps_handler *h, envid_t veid, char *root,
 		if (exec_mode == MODE_EXEC && argv != NULL) {
 			execvep(argv[0], argv, envp);
 		} else {
-			execve("/bin/bash", argv != NULL ? argv : argv_bash,
-					envp);
-			execve("/bin/sh", argv != NULL ? argv : argv_bash,
-					envp);
+			if (argv == NULL)
+				argv = def_argv;
+			argv[0] = "/bin/bash";
+			execve(argv[0], argv, envp);
+			argv[0] = "/bin/sh";
+			execve(argv[0], argv, envp);
 		}
 		ret = VZ_FS_BAD_TMPL;
 env_err:
@@ -326,8 +326,8 @@ err:
  * @param timeout	execution timeout, 0 - unlimited.
  * @return		0 on success.
  */
-int vps_exec(vps_handler *h, envid_t veid, char *root, int exec_mode,
-	char *const argv[], char *const envp[], char *std_in, int timeout)
+int vps_exec(vps_handler *h, envid_t veid, const char *root, int exec_mode,
+	char *argv[], char *const envp[], char *std_in, int timeout)
 {
 	int pid, ret;
 
@@ -337,6 +337,9 @@ int vps_exec(vps_handler *h, envid_t veid, char *root, int exec_mode,
 		logger(-1, 0, "Container is not running");
 		return VZ_VE_NOT_RUNNING;
 	}
+	fflush(stderr);
+	fflush(stdout);
+	/* Extra fork to skip UBC limit applying to the current process */
 	if ((pid = fork()) < 0) {
 		logger(-1, errno, "Can not fork");
 		return VZ_RESOURCE_ERROR;
@@ -350,8 +353,8 @@ int vps_exec(vps_handler *h, envid_t veid, char *root, int exec_mode,
 	return ret;
 }
 
-int _real_execFn(vps_handler *h, envid_t veid, char *root, execFn fn, void *data,
-	int flags)
+int _real_execFn(vps_handler *h, envid_t veid, const char *root,
+		execFn fn, void *data, int flags)
 {
 	int ret, pid;
 
@@ -381,8 +384,8 @@ env_err:
 	return ret;
 }
 
-int vps_execFn(vps_handler *h, envid_t veid, char *root, execFn fn, void *data,
-	int flags)
+int vps_execFn(vps_handler *h, envid_t veid, const char *root,
+		execFn fn, void *data, int flags)
 {
 	int pid, ret;
 
@@ -392,6 +395,9 @@ int vps_execFn(vps_handler *h, envid_t veid, char *root, execFn fn, void *data,
 		logger(-1, 0, "Container is not running");
 		return VZ_VE_NOT_RUNNING;
 	}
+	fflush(stderr);
+	fflush(stdout);
+	/* Extra fork to skip UBC limit applying to the current process */
 	if ((pid = fork()) < 0) {
 		logger(-1, errno, "Can not fork");
 		return VZ_RESOURCE_ERROR;
@@ -415,8 +421,8 @@ int vps_execFn(vps_handler *h, envid_t veid, char *root, execFn fn, void *data,
  * @param timeout	execution timeout, 0 - unlimited.
  * @return		0 on success.
  */
-int vps_exec_script(vps_handler *h, envid_t veid, char *root,
-	char *const argv[], char *const envp[], const char *fname, char *func,
+int vps_exec_script(vps_handler *h, envid_t veid, const char *root,
+	char *argv[], char *const envp[], const char *fname, char *func,
 	int timeout)
 {
 	int len, ret;
@@ -426,19 +432,18 @@ int vps_exec_script(vps_handler *h, envid_t veid, char *root,
 		return -1;
 	logger(1, 0, "Running container script: %s", fname);
 	ret = vps_exec(h, veid, root, MODE_BASH, argv, envp, script, timeout);
-	if (script != NULL)
-		free(script);
+	free(script);
 	return ret;
 }
 
 
 int vps_run_script(vps_handler *h, envid_t veid, char *script, vps_param *vps_p)
 {
-	int is_run;
+	int is_run, is_mounted;
 	int rd_p[2], wr_p[2];
 	int ret, retry;
 	char *argv[2];
-	char *root = vps_p->res.fs.root;
+	const char *root = vps_p->res.fs.root;
 
 	if (!stat_file(script))	{
 		logger(-1, 0, "Script not found: %s", script);
@@ -458,7 +463,7 @@ int vps_run_script(vps_handler *h, envid_t veid, char *script, vps_param *vps_p)
 		return VZ_FS_NOPRVT;
 	}
 	if (!(is_run = vps_is_run(h, veid))) {
-		if (!vps_is_mounted(root)) {
+		if (!(is_mounted = vps_is_mounted(root))) {
 			if ((ret = fsmount(veid, &vps_p->res.fs,
 				&vps_p->res.dq )))
 			{
@@ -480,7 +485,8 @@ int vps_run_script(vps_handler *h, envid_t veid, char *script, vps_param *vps_p)
 		retry = 0;
 		while (retry++ < 10 && vps_is_run(h, veid))
 			usleep(500000);
-		fsumount(veid, root);
+		if (!is_mounted)
+			fsumount(veid, root);
 	}
 	return ret;
 }
