@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,9 +28,8 @@
 #include "types.h"
 #include "config.h"
 
-/* The below two lines are needed to link vzsplit with libvzctl */
-#include <logger.h>
-LOG_DATA
+#include "logger.h"
+#include "util.h"
 
 /* #include <linux/magic.h>
  * Instead of including a file which is not present in all environments,
@@ -53,9 +52,9 @@ LOG_DATA
 
 /* maximal per system values */
 #define MAX_TOTAL_PIDS	8000
-#define MAXIPTENT	2000 
+#define MAXIPTENT	2000
 
-/* minimal per VE values */
+/* minimal per CT values */
 #define MINPROC		30
 #define MINTCPBUF	65536
 #define MINDGRAMBUF	32768
@@ -65,14 +64,14 @@ LOG_DATA
 #define MIN_PGLOCK	4
 #define MIN_GUARPG	1024
 
-/* maximal per VE values */
+/* maximal per CT values */
 #define PRIVVM_PVE	0.6 / 4096
 #define IPTENT_PVE	200
 #define FLOCK_PVE	1000
 #define PTY_PVE		512
 
 /* default diskspace values */
-#define HOST_DS		10737418240ULL /* 10 GB */
+#define HOST_DS		10485760ULL /* 10 GB */
 #define DEF_DS		225280
 #define HOST_DI		100000
 #define DEF_DI		88000
@@ -154,7 +153,7 @@ struct par_limits {
 struct par_limits params[NUMUBC];
 
 /* Global variables */
-unsigned long long mem_total, low_total, swap_total, ds_total, di_total; 
+unsigned long long mem_total, low_total, swap_total, ds_total, di_total;
 long pagesize, proc_calc;
 int num_ve, ve_allowed, osl;
 
@@ -170,25 +169,27 @@ float	k_privvm[MAX_SL]	= {6, 3, 1.5};
 int	k_pglock[MAX_SL]	= {10, 3, 1};
 int	k_msl[MAX_SL]		= {10485760, 2097152, 0};
 
-char *level_string[MAX_SL+1] = {
-"Free resource distribution. Any parameters may be increased\0",
-"Normal resource distribution. Secondary parameters may be increased\0",
-"Partial resource shortage. Auxiliary parameters may be increased\0",
-"Overall resource shortage. Please, do not change any parameters!\0"
+char *level_string[MAX_SL + 1] = {
+	"Free resource distribution. Any parameters may be increased",
+	"Normal resource distribution. Secondary parameters may be increased",
+	"Partial resource shortage. Auxiliary parameters may be increased",
+	"Overall resource shortage. Please, do not change any parameters!"
 };
 
-void usage()
+void usage(int rc)
 {
-	fprintf(stderr, "Usage: vzsplit [-f config_name] "
-			"| [-n numves] | [-s swap_size\n"
+	fprintf(rc ? stderr : stdout, "Usage: vzsplit [-f config_name] "
+		"| [-n numves] | [-s swap_size]\n"
 		"\t -f specified config name\n"
-		"\t -n specified number of VEs\n"
+		"\t -n specified number of containers\n"
 		"\t -s specified swap in Kbytes\n");
+	exit(rc);
 }
 
 void header(FILE *fp)
 {
-	fprintf(fp,"# Configuration file generated by vzsplit for %d VEs\n"
+	fprintf(fp,
+		"# Configuration file generated by vzsplit for %d containers\n"
 		"# on HN with total amount of physical mem %llu Mb\n"
 		"# low memory %llu Mb, swap size %llu Mb, Max treads %lu\n"
 		"# Resourse commit level %d:\n# %s\n",
@@ -205,15 +206,16 @@ int get_cpupower(int *cpuunits)
 	FILE *fd;
 	char str[1024];
 	int val, total = 0;
-	
+
 	*cpuunits = DEF_CPUUNITS;
 	if ((fd = fopen(PROCCPU, "r")) == NULL) {
-		fprintf(stderr, "Cannot open " PROCCPU "\n");
+		logger(-1, errno, "Cannot open " PROCCPU);
 		return 1;
 	}
 	while (fgets(str, sizeof(str), fd))
 		if (sscanf(str, "bogomips\t: %u", &val) == 1)
 			total += val;
+	fclose(fd);
 	if (total) {
 		total *= 25;
 		*cpuunits = total / (num_ve + 1);
@@ -226,11 +228,10 @@ int lconv(char *name)
 {
 	int i, cpuunits;
 	FILE *fp;
-	
+
 	if (name != NULL) {
 		if ((fp = fopen(name, "w")) == NULL) {
-			fprintf(stderr, "Unable to create %s: %s\n", name,
-					strerror(errno));
+			logger(-1, errno, "Unable to create %s", name);
 			return 1;
 		}
 	} else {
@@ -252,7 +253,7 @@ int lconv(char *name)
 	fprintf(fp, "CPUUNITS=\"%d\"\n", cpuunits);
 	if (name) {
 		fclose(fp);
-		fprintf(stderr, "Config %s was created\n", name);
+		logger(-1, 0, "Config %s was created", name);
 	}
 	return 0;
 }
@@ -283,7 +284,7 @@ int calculate_values()
 		avnproc = kmem / k_kpp[osl];
 		if (avnproc < MINPROC / 2)
 			continue;
-		
+
 		if (numproc <  avnproc)
 			numproc = avnproc;
 		if ((numproc < 2 * avnproc) && (osl < MAX_SL))
@@ -321,8 +322,8 @@ int calculate_values()
 		} while  ((rest < 0) && (numproc > MINPROC));
 
 		if (rest < 0) {
-			fprintf(stderr, "Fatal resource shortage, "
-					"try to decrease the number of VEs\n");
+			logger(-1, 0, "Fatal resource shortage, try to "
+					"decrease the number of containers");
 			return 1;
 		}
 		params[DGRAM].lim = params[DGRAM].bar = MINDGRAMBUF;
@@ -334,14 +335,14 @@ int calculate_values()
 		if (rest > delta) {
 			params[TCPRCV].lim = MINTCPBUF + delta;
 			kmem += rest - delta;
-		} else 
+		} else
 			params[TCPRCV].lim = MINTCPBUF + rest;
 	} else {
 		delta = k_sock2[sl] * numproc;
 		rest = tcpbuf - delta;
 		params[TCPSND].lim = params[TCPRCV].lim = CHECK_LIMIT(tcpbuf);
 		params[TCPSND].bar = params[TCPRCV].bar = CHECK_LIMIT(rest);
-		params[DGRAM].lim = params[DGRAM].bar = 
+		params[DGRAM].lim = params[DGRAM].bar =
 			params[SOCKOTH].bar = CHECK_LIMIT(rest / 2);
 		params[SOCKOTH].lim = CHECK_LIMIT(rest / 2 + delta);
 	}
@@ -365,7 +366,7 @@ int calculate_values()
 	params[NFILE].lim = params[NFILE].bar = CHECK_LIMIT(numfile);
 
 	dcache = numfile * k_dcache[sl];
-	params[DCACHE].lim = CHECK_LIMIT(dcache); 
+	params[DCACHE].lim = CHECK_LIMIT(dcache);
 	params[DCACHE].bar = CHECK_LIMIT(dcache / DCACHE_DELTA);
 
 	numflock = numfile * NFLOCK_NFILE;
@@ -378,22 +379,23 @@ int calculate_values()
 	if (lockedpages < MIN_PGLOCK)
 		lockedpages = MIN_PGLOCK;
 	params[LOCKPG].lim = params[LOCKPG].bar = CHECK_LIMIT(lockedpages);
-	
+
 	numpty = numproc * PTY_PROC;
 	if (numpty < MIN_PTY)
 		numpty = MIN_PTY;
 	if (numpty > PTY_PVE)
 		numpty = PTY_PVE;
-	
+
 	params[NPTY].lim = params[NPTY].bar = numpty;
-	
+
 	params[NSIGINFO].lim = params[NSIGINFO].bar = numsiginfo[sl];
 	params[NPHYPG].bar = 0; params[NPHYPG].lim = MAXVAL;
 
 	privvm = guarpg * k_privvm[sl];
 	if (privvm > PRIVVM_PVE * mem_total) {
 		privvm = PRIVVM_PVE * mem_total;
-		guarpg = privvm;
+		if (guarpg > privvm)
+			guarpg = privvm;
 	}
 	params[PRIVVMPG].bar = CHECK_LIMIT(privvm);
 	params[PRIVVMPG].lim = CHECK_LIMIT(privvm * PRIVVM_DELTA);
@@ -428,8 +430,7 @@ char * get_ve_private()
 	vps_parse_config(0, GLOBAL_CFG, param, NULL);
 	ve_private = param->res.fs.private_orig;
 
-	if (ve_private == NULL)
-	{
+	if (ve_private == NULL) {
 		free_vps_param(param);
 		return NULL;
 	}
@@ -455,25 +456,27 @@ int check_disk_space() {
 
 	ve_private = get_ve_private();
 	if (ve_private == NULL) {
-		fprintf(stderr, "WARNING: unable to get VE_PRIVATE value "
-				"from " GLOBAL_CFG ".\n");
+		logger(-1, 0, "WARNING: unable to get VE_PRIVATE value "
+				"from " GLOBAL_CFG ".");
 		nofs = 1;
 	}
 	else {
-		if (statfs (ve_private, &statfs_buf) < 0) {
-			fprintf(stderr, "WARNING: statfs on %s failed: %s.\n",
-				ve_private, strerror(errno));
+		if (statfs(ve_private, &statfs_buf) < 0) {
+			logger(-1, errno, "WARNING: statfs on %s failed",
+				ve_private);
 			nofs = 1;
 		}
 	}
 
 	if (nofs == 1) {
-		fprintf(stderr, "Default disk space values to be used.\n\n");
+		logger(-1, 0, "Default disk space values to be used.\n");
 		ds_total = 0; di_total = 0;
 		return retval;
 	}
 
 	ds_total = statfs_buf.f_blocks;
+	/* convert to Kbytes */
+	ds_total *= (statfs_buf.f_bsize / 1024);
 	di_total = statfs_buf.f_files;
 
 	if (statfs_buf.f_type == REISERFS_SUPER_MAGIC) {
@@ -482,11 +485,11 @@ int check_disk_space() {
 		noinodes = 1;
 	}
 
-	if (ds_total / 2 < HOST_DS / statfs_buf.f_bsize) {
+	if (ds_total / 2 < HOST_DS) {
 		rec = 1;
 		ds_total /= 2;
 	} else
-		ds_total -= HOST_DS / statfs_buf.f_bsize;
+		ds_total -= HOST_DS;
 
 	if (noinodes != 1) {
 		if (di_total / 2 < HOST_DI) {
@@ -496,12 +499,12 @@ int check_disk_space() {
 			di_total -= HOST_DI;
 	}
 	if (rec)
-		fprintf(stderr, "WARNING: Recommended minimal size "
-				"of partition holding %s is 20Gb!\n",
+		logger(-1, 0, "WARNING: Recommended minimal size "
+			"of partition holding %s is 20Gb!",
 				ve_private);
 
-	ve_ds = ds_total / (DEF_DS);
-	ve_di = di_total / (DEF_DI);
+	ve_ds = ds_total / DEF_DS;
+	ve_di = di_total / DEF_DI;
 
 	if (ve_ds < num_ve) {
 		retval = 1;
@@ -514,12 +517,12 @@ int check_disk_space() {
 			ve_allowed = ve_di;
 	}
 	if (retval == 1) {
-		fprintf(stderr, "WARNING: partition holding %s do not "
-				"have space required for %d VEs\n"
-				"The maximum allowed value is %d\n",
+		logger(-1, 0, "WARNING: partition holding %s do not "
+				"have space required for %d containers\n"
+				"The maximum allowed value is %d",
 				ve_private, num_ve, ve_allowed);
-		fprintf(stderr, "Default disk space values "
-				"will be used\n\n");
+		logger(-1, 0, "Default disk space values "
+				"will be used\n");
 		ds_total = 0; di_total = 0;
 	}
 
@@ -546,15 +549,19 @@ int main(int argc, char **argv)
 	low_total = 0;
 	swap_total = 0;
 
+	init_log(NULL, 0, 1, 0, 0, "vzsplit");
+
 	while ((opt = getopt(argc, argv, "f:n:s:h")) > 0) {
 		switch(opt) {
 		case 'f':
 			len = strlen(optarg) + strlen(VPS_CONF_DIR) +
 				strlen("ve-.conf-sample");
-			name = (char*)malloc(len + 1);
+			name = (char *)vz_malloc(len + 1);
+			if (!name)
+				exit(1);
 			sprintf(name, VPS_CONF_DIR "ve-%s.conf-sample", optarg);
 			if (!stat(name, &st)) {
-				fprintf(stderr,"File %s already exist\n",
+				logger(-1, 0, "File %s already exist",
 					name);
 				exit(1);
 			}
@@ -562,49 +569,48 @@ int main(int argc, char **argv)
 		case 'n':
 			num_ve = strtol(optarg, &tail, 10);
 			if (*tail != '\0') {
-				fprintf(stderr, "Invalid argument "
-						"for -n: %s\n",	optarg);
+				logger(-1, 0, "Invalid argument "
+						"for -n: %s", optarg);
 				exit(1);
 			}
 			break;
 		case 's':
 			if (optarg[0] == '-') {
-				fprintf(stderr, "Negative value for -s\n");
+				logger(-1, 0, "Negative value for -s");
 				exit(-1);
 			}
 			swp = 1;
 			swap_total = strtoll(optarg, &tail, 10);
 			swap_total <<= 10;
 			if (*tail != '\0') {
-				fprintf(stderr, "Invalid argument "
-						"for -s: %s\n",	optarg);
+				logger(-1, 0, "Invalid argument "
+						"for -s: %s", optarg);
 				exit(1);
 			}
 			break;
 		case 'h':
+			usage(0);
 		default	:
-			usage();
-			exit(0);
+			usage(1);
 		}
 	}
 	if (optind < argc) {
-		usage();
-		exit(1);
+		usage(1);
 	}
 	if (num_ve == -1) {
-		printf("Enter the number of VEs: ");
+		printf("Enter the number of containers: ");
 		if (scanf("%d", &num_ve) != 1) {
-			fprintf(stderr, "Invalid value.\n");
+			logger(-1, 0, "Invalid value.");
 			exit(1);
 		}
 	}
 	if (num_ve < 1) {
-		fprintf(stderr, "Incorrect value for number of VEs.\n");
+		logger(-1, 0, "Incorrect value for number of containers.");
 		exit(1);
 	}
-	
+
 	if ((fd = fopen(PROCMEM, "r")) == NULL) {
-		fprintf(stderr, "Cannot open " PROCMEM"\n");
+		logger(-1, errno, "Cannot open " PROCMEM);
 		exit(1);
 	}
 
@@ -617,22 +623,24 @@ int main(int argc, char **argv)
 			swap_total = val << 10;
 
 	fclose(fd);
+	if (low_total == 0)
+		low_total = mem_total;
 	if (mem_total < SYSRSRV) {
-		fprintf(stderr, "At least 128 Mb of RAM should be "
-				"installed on Hardware Node\n");
+		logger(-1, 0, "At least 128 Mb of RAM should be "
+				"installed on Hardware Node");
 		exit(1);
 	}
 
 	if (swap_total > 2 * mem_total)
-		fprintf(stderr, "The optimal swap space size is %llu Mb, "
-				"twice bigger than the RAM size\n\n", 
+		logger(-1, 0, "The optimal swap space size is %llu Mb, "
+				"twice bigger than the RAM size\n",
 				(2 * mem_total) >> 20);
 	ve_allowed = num_ve;
 	retval = 0;
-	
+
 	if (((mem_total + swap_total - SYSRSRV) / num_ve) < MEMPERVE) {
-		fprintf(stderr, "On node with %llu Mb of memory (RAM + swap) "
-				"%d VEs can not be allocated\n", 
+		logger(-1, 0, "On node with %llu Mb of memory (RAM + swap) "
+				"%d containers can not be allocated",
 				(mem_total + swap_total) >> 20, num_ve);
 		ve_allowed = (mem_total + swap_total - SYSRSRV) / MEMPERVE;
 		retval = 1;
@@ -640,24 +648,24 @@ int main(int argc, char **argv)
 
 	if (((low_total - SYSRSRV)/ ve_allowed) < LOWPERVE) {
 		int ve_low;
-		
-		fprintf(stderr, "On node with %llu Mb of Low Memory "
-				"%d VEs can not be allocated\n", 
+
+		logger(-1, 0, "On node with %llu Mb of Low Memory "
+				"%d containers can not be allocated",
 				low_total >> 20, num_ve);
-		ve_low = (low_total - SYSRSRV) / LOWPERVE; 
+		ve_low = (low_total - SYSRSRV) / LOWPERVE;
 		if (ve_low < ve_allowed)
 			ve_allowed = ve_low;
 		retval = 1;
 	}
 
 	if (retval != 0) {
-		fprintf(stderr, "The maximum allowed value is %d\n",
+		logger(-1, 0, "The maximum allowed value is %d",
 				ve_allowed);
 		exit(retval);
 	}
 
 	if ((fd = fopen(PROCTHREADS, "r")) == NULL) {
-		fprintf(stderr, "Cannot open " PROCTHREADS "\n");
+		logger(-1, errno, "Cannot open " PROCTHREADS);
 		exit(1);
 	}
 	if (fgets(str, sizeof(str), fd))
@@ -669,12 +677,12 @@ int main(int argc, char **argv)
 
 	if ((pagesize = sysconf(_SC_PAGE_SIZE)) == -1)
 		pagesize = 4096;
-	
-	if (low_total > 2 << 30) {
+
+	if (low_total > 2U << 30) {
 		if (proc_calc > 2 * MAX_TOTAL_PIDS)
 			proc_calc = 2 * MAX_TOTAL_PIDS;
 	} else if (proc_calc > MAX_TOTAL_PIDS)
-			proc_calc = MAX_TOTAL_PIDS; 
+			proc_calc = MAX_TOTAL_PIDS;
 
 	if (calculate_values())
 		exit(1);
@@ -682,4 +690,3 @@ int main(int argc, char **argv)
 	retval = lconv(name);
 	exit(retval);
 }
-
